# -*- coding: utf-8 -*-
import objectpack

from m3_ext.ui import render_component
from m3_ext.ui import all_components as ext
from m3_ext.ui.containers.grids import ExtGridGroupingView
from m3_ext.ui.misc import store as ext_store
from m3_ext.ui.misc.store import ExtJsonWriter


class GridPanel(ext.ExtPanel):
    u"""
    Панель, имеющая grid_url, по которому получает грид и вставляет в себя
    """

    grid_url = None
    template = 'grid-panel.js'
    grid_class = ext.ExtObjectGrid

    def __init__(self, *args, **kwargs):
        # в инитах затирается темплейт
        t = self.template
        super(GridPanel, self).__init__(*args, **kwargs)
        self.template = t

    @staticmethod
    def _create_grid(pack, columns, grid_cls, group_by=None, **kwargs):
        u"""
        Метод возвращает грид
        :param grid_cls: класс грида
        :type grid_cls: ExtObjectGrid
        :param str group_by: темплейт для группирования
        """
        grid = grid_cls(
            region='north',
            force_fit=False,
            auto_scroll=True,
            auto_width=False,
            cls='word-wrap-grid',
            **kwargs
        )
        grid.flex = 1
        grid.sm = ext.ExtGridCellSelModel()
        grid.allow_paging = False

        # создание колонок
        cc = objectpack.ColumnsConstructor()

        # если грид с группирующими строками
        if group_by:
            grid.store = ext_store.ExtGroupingStore(
                url=grid.get_store().url,
                total_property='total',
                root='rows',
                auto_load=True
            )
            grid.store.reader = ext_store.ExtJsonReader(
                total_property='total',
                root='rows')
            grid.view = ExtGridGroupingView(
                group_text_template=group_by,
                force_fit=grid.force_fit
            )

            def configure_reader(col):
                grid.store.reader.set_fields(col['data_index'])
        else:
            configure_reader = lambda col: None

        def populate(root, columns):
            for c in columns:
                sub_cols = c.pop('columns', None)
                params = {}
                params.update(c)
                params['header'] = unicode(params.pop('header', ''))
                # TODO - сомнительная проверка
                if sub_cols is not None:
                    if sub_cols:
                        new_root = cc.BandedCol(**params)
                        root.add(new_root)
                        populate(new_root, sub_cols)
                else:
                    configure_reader(c)
                    root.add(cc.Col(**params))

        populate(cc, columns)
        cc.configure_grid(grid)

        # атрибуты грида
        grid.row_id_name = pack.row_id_name
        grid.column_param_name = pack.column_param_name
        grid.editor = True
        grid.store.writer = ExtJsonWriter(write_all_fields=False)
        grid.url_data = pack.get_rows_url()

        # список исключений для make_read_only
        grid._mro_exclude_list = []

        return grid

    @classmethod
    def configure_grid(cls, grid, params):
        u"""Конфигурирование грида"""

        pack = params['pack']
        # обычное поведение
        grid.sm = ext.ExtGridCellSelModel()
        grid.row_id_name = pack.row_id_name
        grid.url_data = pack.get_rows_url()
        # кнопка "Обновить" не участвует в блокировке грида при make_read_only
        grid._mro_exclude_list.append(grid.top_bar.button_refresh)

        return grid

    @classmethod
    def create_grid(cls, pack, columns, **kwargs):
        grid = cls._create_grid(pack, columns, cls.grid_class, **kwargs)
        return grid

    def pre_render(self):
        assert self.grid_url, u'grid url must be defined in gridpanel!'
        self._put_config_value('grid_url', self.grid_url)

    def render(self):
        self.pre_render()
        self.cmp_code = super(GridPanel, self).render()
        return render_component(self)


class FilterPanel(ext.ExtPanel):
    u"""
    Панель, умеющая fireevent'ить об изменении контрола,
    входящего в список filters
    """

    def __init__(self, *args, **kwargs):
        super(FilterPanel, self).__init__(*args, **kwargs)
        self.filters = []

    def configure_redirect(self, control, event):
        u"""
        Связывает текущую панель с контролом control
        посредством fireevent'а event
        """
        self._listeners["changed"] = """
            function(params) {
                Ext.getCmp('%s').fireEvent('%s', params)
            }""" % (control.client_id, event)

    def configure_events(self):
        u"""
        Дополнение рендера fireevent'ом изменения значения контролов,
        входящих в список filters панели
        """
        for control in self.filters:
            # определяется что за контрол и приделываются разные events
            if isinstance(control, objectpack.ui.ComboBoxWithStore):
                control.events = [
                    ('select', 'makeSimpleHandler'),
                ]
                """
                ..code::

                    control.events = ('custom_event', '(function(ctl, name){
                        custom handler })')
                """
            if isinstance(control, ext.ExtDictSelectField):
                control.events = [('change', 'makeSimpleHandler')]
            if isinstance(control, ext.ExtDateField):
                control.events = [('select', 'makeSimpleHandler')]

    def render(self):
        self.configure_events()
        self.pre_render()
        self.cmp_code = super(FilterPanel, self).render()
        # где-то в инитах затирается темплейт, опишем еще раз
        self.template = 'filter-panel.js'
        return render_component(self)


class BaseGridWindow(objectpack.BaseWindow):
    u"""
    Базовое окно с фильтрующей панелью и панелью с гридом
    Клиент получает панель с гридом с сервера при изменении
    полей фильтрации на фильтрующей панели
    """
    grid_panel_cls = GridPanel  # панель грида

    def _init_grid_panel(self):
        u"""Панель для грида."""
        self.grid_panel = self.grid_panel_cls(
            region='center', layout='hbox', flex=1,
            layout_config={"align": "stretch"}
        )

    def _init_filter_panel(self):
        u"""Панель с фильтрами."""
        self.filter_cnt = FilterPanel(
            body_cls='x-window-mc',
            padding='5px 7px',
            border=False,
            body_border=False,
            region='north',
            layout='column',
            height=35,
        )

    def _init_components(self):
        super(BaseGridWindow, self)._init_components()
        self._init_grid_panel()
        self._init_filter_panel()
        # связываются две панели
        self.filter_cnt.configure_redirect(self.grid_panel, 'reloadgrid')
        # добавление фильтрующем панели в список исключения из read_only
        self._mro_exclude_list.append(self.filter_cnt)

    def _do_layout(self):
        super(BaseGridWindow, self)._do_layout()
        self.layout = 'border'
        self.maximizable = self.maximized = True
        self.minimizable = True
        self.width, self.min_width = 870, 870
        self.height = 600
        self.grid_panel.layout_config['align'] = 'stretch'

        # добавление фильтрующих полей в фильтрующую панель
        assert self.filter_cnt.filters, (
            u'Необходимо добавить элементы в список фильтров '
            u'(self.filter_cnt.filters)!')

        self.items.extend([
            self.filter_cnt, self.grid_panel
        ])

    def set_params(self, params):

        super(BaseGridWindow, self).set_params(params)
        self.template_globals = 'base-grid-window.js'

        self.pack = params["grid_pack"]

        # урл получения грида для панели с гридом
        self.grid_panel.grid_url = self.pack.get_grid_action_url()

        # атрибуты окна, например для cell select
        self.column_name = self.pack.column_param_name
        self.column_prefix = self.pack.column_param_name

        # заголовок окна
        self.title = self.pack.title
